<?php

namespace Ig\IgCookies\Controller;

use TYPO3\CMS\Core\Utility\GeneralUtility;
use Psr\Http\Message\ResponseInterface;
use TYPO3\CMS\Core\Localization\LanguageServiceFactory;
use TYPO3\CMS\Core\Http\JsonResponse;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;

class MainController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{
    protected $languageFile = 'LLL:EXT:ig_cookies/Resources/Private/Language/locallang.xlf';
    protected $labelKeys = ['title', 'description'];

    /**
     * action cookie: display info to accept cookies
     */
    public function consentAction(): ResponseInterface
    {
        $site = $GLOBALS['TYPO3_REQUEST']->getAttribute('site');
        $rootPageUid = $site->getRootPageId();
         $this->uriBuilder
             ->reset()
             ->setTargetPageUid($rootPageUid)
             ->setTargetPageType(1690140404);
         // v12
         if (method_exists($this, 'getRequest')) {
             $this->uriBuilder->setRequest($this->getRequest());
         }
         $configUrl = $this->uriBuilder->build();
        
        $this->view->assign('configUrl', $configUrl);

        return $this->htmlResponse($this->view->render());
    }


    /**
     * config action
     */
    public function configAction(): ResponseInterface
    {
        $this->initServices();
        $this->translations = [
            'alwaysRequired' => LocalizationUtility::translate('LLL:EXT:ig_cookies/Resources/Private/Language/locallang.xlf:always-required'),
            'alwaysUnlockBefore' => LocalizationUtility::translate('LLL:EXT:ig_cookies/Resources/Private/Language/locallang.xlf:always-unlock.before'),
            'alwaysUnlockAfter' => LocalizationUtility::translate('LLL:EXT:ig_cookies/Resources/Private/Language/locallang.xlf:always-unlock.after'),
        ];
        $response = [
            'purposes' => $this->purposesArray,
            'services' => $this->servicesArray,
            'translations' => $this->translations,
        ];

        //return json_encode($response);
        return new JsonResponse($response);
    }

    
    public function getTranslatedLabels($purpose, &$arr)
    {
        foreach ($this->labelKeys as $label) {
            if ($purpose[$label]) {
                $ll = trim($purpose[$label]);
                if (substr($ll, 0, 8) == 'LLL:EXT:') {
                    $arr[$label] = LocalizationUtility::translate($ll) ?? '';
                } else if (strpos($ll, '.') && !strpos($ll, ' ')) {
                    $arr[$label] = LocalizationUtility::translate($this->languageFile . ':' . $ll) ?? '';
                } else {
                    $arr[$label] = $ll;
                }
            }
        }
    }
    public function translateArray(&$arr)
    {
        foreach ($arr as $name => &$value) {
            if (is_array($value)) {
                $this->translateArray($value);
            } elseif ($value) {
                $ll = trim($value);
                if (substr($ll, 0, 8) == 'LLL:EXT:') {
                    $value = LocalizationUtility::translate($ll) ?? '';
                }
            }
        }
    }

    protected function initServices(): void
    {
        $this->purposesArray = [];
        $this->servicesArray = [];
        $purposes = array_filter($this->settings['purposes']);  //all purposes
        $activePurposes = [];
        $usedPurposes = [];
        $this->services = array_filter($this->settings['services']);
        if ($this->settings['languageFile'] ?? false) {
            $this->languageFile = $this->settings['languageFile'];
        }

        foreach ($this->services as $name => &$service) {
            
            $service['hidden'] = filter_var($service['hidden'] ?? '', FILTER_VALIDATE_BOOLEAN);
            if ($service['hidden']) {
                continue;
            }
            $service['contextualConsentOnly'] = filter_var($service['contextualConsentOnly'] ?? '', FILTER_VALIDATE_BOOLEAN);
            $this->translateArray($service);
            if (!isset($service['policyText'])) {
                $service['policyText'] = LocalizationUtility::translate($this->languageFile . ':serviceContextualButton.moreText') ?? '';
            }
            if (!isset($service['sorting'])) {
                $service['sorting'] = 0;
            }
            $service['name'] = $name;
            $this->servicesArray[] = $service;
            $purposeName = $service['purpose'];
            if (!isset($activePurposes[$purposeName]) && isset($purposes[$purposeName])) {
                $activePurposes[$purposeName] = true;
            }
        }
        usort($this->servicesArray, function($a, $b) {
            return $a['sorting'] - $b['sorting'];
        });

        // prepare active/used purposes
        foreach ($purposes as $name => $purpose) {
            if(isset($activePurposes[$name])) {
                $purpose['name'] = $name;
                if (!isset($purpose['sorting'])) {
                    $purpose['sorting'] = 0;
                }
                $purpose['required'] = filter_var($purpose['required'] ?? '', FILTER_VALIDATE_BOOLEAN, FILTER_NULL_ON_FAILURE);
                $this->translateArray($purpose);
                $this->purposesArray[] = $purpose;
            }
        }
        usort($this->purposesArray, function($a, $b) {
            return $a['sorting'] - $b['sorting'];
        });
    }



}
